@@ -1,7 +1,7 @@
 /* Trampoline construction */
 
 /*
- * Copyright 1995-1999, 2001-2003 Bruno Haible, <haible@clisp.cons.org>
+ * Copyright 1995-1999, 2001-2006 Bruno Haible, <bruno@clisp.org>
  *
  * This is free software distributed under the GNU General Public Licence
  * described in the file COPYING. Contact the author if you don't have this
@@ -20,16 +20,16 @@
 #define __hppanew__  /* New trampoline, just a closure. */
 #endif
 #endif
-#if defined(__rs6000__)
-#if defined(__NetBSD__)
-#define __rs6000netbsd__
-#else
+#if defined(__powerpc__) && !defined(__powerpc64__)
 #if !defined(_AIX)
-#define __rs6000sysv4__  /* SysV.4 ABI, real machine code. */
+#define __powerpcsysv4__  /* SysV.4 ABI, real machine code. */
 #else
-#define __rs6000aix__  /* AIX ABI, just a closure. */
+#define __powerpcaix__  /* AIX ABI, just a closure. */
 #endif
 #endif
+#if defined(__powerpc64__)
+/* The only ABI on powerpc64 known so far is the AIX ABI. */
+#define __powerpc64aix__  /* AIX ABI, just a closure. */
 #endif
 #if defined(__hppanew__)
 /*
@@ -44,7 +44,7 @@ extern void tramp_r (); /* trampoline prototype */
 #define CODE_EXECUTABLE
 #endif
 #endif
-#if defined(__rs6000aix__) || defined(__ia64__)
+#if defined(__powerpcaix__) || defined(__powerpc64aix__) || defined(__ia64__)
 /*
  * A function pointer is a pointer to a data area whose first word contains
  * the actual address of the function.
@@ -104,10 +104,8 @@ extern void (*tramp_r) (); /* trampoline prototype */
 #ifdef HAVE_GETPAGESIZE
 #ifdef __cplusplus
 extern "C" RETGETPAGESIZETYPE getpagesize (void);
-#elif defined(__STDC__)
-extern RETGETPAGESIZETYPE getpagesize (void);
 #else
-extern RETGETPAGESIZETYPE getpagesize ();
+extern RETGETPAGESIZETYPE getpagesize (void);
 #endif
 #else
 #ifdef HAVE_SYS_PARAM_H
@@ -206,7 +204,7 @@ extern RETGETPAGESIZETYPE getpagesize ();
 #include <sys/syslocal.h>
 #endif
 /* Inline assembly function for instruction cache flush. */
-#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__rs6000sysv4__) || defined(__rs6000netbsd__) || defined(__convex__)
+#if defined(__sparc__) || defined(__sparc64__) || defined(__alpha__) || defined(__hppaold__) || defined(__powerpcsysv4__) || defined(__convex__)
 #ifdef __GNUC__
 extern inline
 #if defined(__sparc__) || defined(__sparc64__)
@@ -218,8 +216,8 @@ extern inline
 #ifdef __hppa__
 #include "cache-hppa.c"
 #endif
-#ifdef __rs6000__
-#include "cache-rs6000.c"
+#if defined(__powerpc__) && !defined(__powerpc64__)
+#include "cache-powerpc.c"
 #endif
 #ifdef __convex__
 #include "cache-convex.c"
@@ -283,18 +281,18 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 4
 #endif
-#ifdef __rs6000sysv4__
+#ifdef __powerpcsysv4__
 #define TRAMP_LENGTH 24
 #define TRAMP_ALIGN 4
 #endif
-#ifdef __rs6000netbsd__
-#define TRAMP_LENGTH 24
-#define TRAMP_ALIGN 4
-#endif
-#ifdef __rs6000aix__
+#ifdef __powerpcaix__
 #define TRAMP_LENGTH 20
 #define TRAMP_ALIGN 4
 #endif
+#ifdef __powerpc64aix__
+#define TRAMP_LENGTH 40
+#define TRAMP_ALIGN 8
+#endif
 #ifdef __m88k__
 #define TRAMP_LENGTH 20
 #define TRAMP_ALIGN 8
@@ -307,6 +305,10 @@ extern void __TR_clear_cache();
 #define TRAMP_LENGTH 32
 #define TRAMP_ALIGN 16
 #endif
+#ifdef __x86_64__
+#define TRAMP_LENGTH 22
+#define TRAMP_ALIGN 16
+#endif
 #ifdef __s390__
 #define TRAMP_LENGTH 22
 #define TRAMP_ALIGN 2
@@ -326,14 +328,7 @@ extern void __TR_clear_cache();
 static char* freelist = NULL;
 #endif
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __TR_function alloc_trampoline_r (__TR_function address, void* data0, void* data1)
-#else
-__TR_function alloc_trampoline_r (address, data0, data1)
-  __TR_function address;
-  void* data0;
-  void* data1;
-#endif
 {
   char* function;
   char* data;
@@ -365,7 +360,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   /* Note: This memory allocation is not multithread-safe. We might need
    * to add special (platform dependent) code for locking.
    * Fortunately, most modern systems where multithread-safety matters
-   * have EXECUTABLE_VIA_MPROTECT, and those which don't (AIX on rs6000 and
+   * have EXECUTABLE_VIA_MPROTECT, and those which don't (AIX on powerpc and
    * HP-UX on hppa) have CODE_EXECUTABLE. Thus no locking code is needed
    * for the moment.
    */
@@ -436,6 +431,22 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   *(long *)  (function + 2)
 #endif
 #ifdef __m68k__
+#ifdef __NetBSD__
+  /* function:
+   *    movel #<data>,a1		22 7C <data>
+   *    jmp <address>			4E F9 <address>
+   *    nop				4E 71
+   */
+  *(short *) (function + 0) = 0x227C;
+  *(long *)  (function + 2) = (long) data;
+  *(short *) (function + 6) = 0x4EF9;
+  *(long *)  (function + 8) = (long) address;
+  *(short *) (function +12) = 0x4E71;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0x227C && \
+  *(unsigned short *) (function + 6) == 0x4EF9 && \
+  *(unsigned short *) (function +12) == 0x4E71
+#else
   /* function:
    *    movel #<data>,a0		20 7C <data>
    *    jmp <address>			4E F9 <address>
@@ -450,6 +461,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   *(unsigned short *) (function + 0) == 0x207C && \
   *(unsigned short *) (function + 6) == 0x4EF9 && \
   *(unsigned short *) (function +12) == 0x4E71
+#endif
 #define tramp_address(function)  \
   *(long *)  (function + 8)
 #define tramp_data(function)  \
@@ -515,37 +527,6 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   *(unsigned int *) (function +16)
 #endif
-#ifdef __s390__
-  /* function:
-
-        bras    %r1,.LTN0_0
-.LT0_0:
-.LC0:
-        .long   0x73554711
-.LC1:
-        .long   0xbabebec0
-.LTN0_0:
-        l       %r0,.LC0-.LT0_0(%r1)
-        l       %r1,.LC1-.LT1_0(%r1)
-        br      %r1
-  */
-  /* What about big endian / little endian ?? */
-  *(unsigned int *)   (function + 0) = 0xA7150006;
-  *(unsigned int *)   (function + 4) = (unsigned int) data;
-  *(unsigned int *)   (function + 8) = (unsigned int) address;
-  *(unsigned int *)   (function +12) = 0x58001000;
-  *(unsigned int *)   (function +16) = 0x58101004;
-  *(unsigned short *) (function +20) = 0x07f1;
-#define is_tramp(function)  \
-  *(int *)            (function + 0) == 0xA7150006 && \
-  *(int *)            (function +12) == 0x58001000 && \
-  *(int *)            (function +16) == 0x58101004 && \
-  *(short *)          (function +20) == 0x07f1
-#define tramp_address(function)  \
-  *(unsigned int *) (function +8)
-#define tramp_data(function)  \
-  *(unsigned int *) (function +4)
-#endif
 #ifdef __mips64old__
   /* function:
    *    dli $2,<data>			3C 02 hi16(hi32(<data>))
@@ -839,40 +820,8 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   ((long *) function)[6]
 #endif
-#ifdef __rs6000sysv4__
-  /* function:
-   *    {liu|lis} 11,hi16(<data>)		3D 60 hi16(<data>)
-   *    {oril|ori} 11,11,lo16(<data>)		61 6B lo16(<data>)
-   *    {liu|lis} 0,hi16(<address>)		3C 00 hi16(<address>)
-   *    {oril|ori} 0,0,lo16(<address>)		60 00 lo16(<address>)
-   *    mtctr 0					7C 09 03 A6
-   *    bctr					4E 80 04 20
-   */
-  *(short *) (function + 0) = 0x3D60;
-  *(short *) (function + 2) = (unsigned long) data >> 16;
-  *(short *) (function + 4) = 0x616B;
-  *(short *) (function + 6) = (unsigned long) data & 0xffff;
-  *(short *) (function + 8) = 0x3C00;
-  *(short *) (function +10) = (unsigned long) address >> 16;
-  *(short *) (function +12) = 0x6000;
-  *(short *) (function +14) = (unsigned long) address & 0xffff;
-  *(long *)  (function +16) = 0x7C0903A6;
-  *(long *)  (function +20) = 0x4E800420;
-#define is_tramp(function)  \
-  *(unsigned short *) (function + 0) == 0x3D60 && \
-  *(unsigned short *) (function + 4) == 0x616B && \
-  *(unsigned short *) (function + 8) == 0x3C00 && \
-  *(unsigned short *) (function +12) == 0x6000 && \
-  *(unsigned long *)  (function +16) == 0x7C0903A6 && \
-  *(unsigned long *)  (function +20) == 0x4E800420
-#define hilo(hiword,loword)  \
-  (((unsigned long) (hiword) << 16) | (unsigned long) (loword))
-#define tramp_address(function)  \
-  hilo(*(unsigned short *) (function +10), *(unsigned short *) (function +14))
-#define tramp_data(function)  \
-  hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
-#endif
-#ifdef __rs6000netbsd__
+#ifdef __powerpcsysv4__
+#ifdef __NetBSD__
   /* function:
    *    {liu|lis} 13,hi16(<data>)		3D A0 hi16(<data>)
    *    {oril|ori} 13,13,lo16(<data>)		61 AD lo16(<data>)
@@ -898,6 +847,33 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   *(unsigned short *) (function +12) == 0x6000 && \
   *(unsigned long *)  (function +16) == 0x7C0903A6 && \
   *(unsigned long *)  (function +20) == 0x4E800420
+#else
+  /* function:
+   *    {liu|lis} 11,hi16(<data>)		3D 60 hi16(<data>)
+   *    {oril|ori} 11,11,lo16(<data>)		61 6B lo16(<data>)
+   *    {liu|lis} 0,hi16(<address>)		3C 00 hi16(<address>)
+   *    {oril|ori} 0,0,lo16(<address>)		60 00 lo16(<address>)
+   *    mtctr 0					7C 09 03 A6
+   *    bctr					4E 80 04 20
+   */
+  *(short *) (function + 0) = 0x3D60;
+  *(short *) (function + 2) = (unsigned long) data >> 16;
+  *(short *) (function + 4) = 0x616B;
+  *(short *) (function + 6) = (unsigned long) data & 0xffff;
+  *(short *) (function + 8) = 0x3C00;
+  *(short *) (function +10) = (unsigned long) address >> 16;
+  *(short *) (function +12) = 0x6000;
+  *(short *) (function +14) = (unsigned long) address & 0xffff;
+  *(long *)  (function +16) = 0x7C0903A6;
+  *(long *)  (function +20) = 0x4E800420;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0x3D60 && \
+  *(unsigned short *) (function + 4) == 0x616B && \
+  *(unsigned short *) (function + 8) == 0x3C00 && \
+  *(unsigned short *) (function +12) == 0x6000 && \
+  *(unsigned long *)  (function +16) == 0x7C0903A6 && \
+  *(unsigned long *)  (function +20) == 0x4E800420
+#endif
 #define hilo(hiword,loword)  \
   (((unsigned long) (hiword) << 16) | (unsigned long) (loword))
 #define tramp_address(function)  \
@@ -905,7 +881,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   hilo(*(unsigned short *) (function + 2), *(unsigned short *) (function + 6))
 #endif
-#ifdef __rs6000aix__
+#ifdef __powerpcaix__
   /* function:
    *    .long .tramp_r
    *    .long .mytoc
@@ -926,6 +902,27 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #define tramp_data(function)  \
   ((long *) function)[3]
 #endif
+#ifdef __powerpc64aix__
+  /* function:
+   *    .quad .tramp_r
+   *    .quad .mytoc
+   *    .quad 0
+   * .mytoc:
+   *    .quad <data>
+   *    .quad <address>
+   */
+  *(long *)  (function + 0) = ((long *) &tramp_r)[0];
+  *(long *)  (function + 8) = (long) (function + 24);
+  *(long *)  (function +16) = 0;
+  *(long *)  (function +24) = (long) data;
+  *(long *)  (function +32) = (long) address;
+#define is_tramp(function)  \
+  ((long *) function)[0] == ((long *) &tramp_r)[0]
+#define tramp_address(function)  \
+  ((long *) function)[4]
+#define tramp_data(function)  \
+  ((long *) function)[3]
+#endif
 #ifdef __m88k__
   /* function:
    *    or.u    #r11,#r0,hi16(<data>)		5D 60 hi16(<data>)
@@ -994,6 +991,67 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   ((long *) function)[2]
 #define tramp_data(function)  \
   ((long *) function)[3]
+#endif
+#ifdef __x86_64__
+  /* function:
+   *    movabsq $<data>,%r10		49 BA <data>
+   *    movabsq $<address>,%rax		48 B8 <address>
+   *    jmp *%rax			FF E0
+   */
+  *(short *) (function + 0) = 0xBA49;
+  *(short *) (function + 2) = (unsigned long) data & 0xffff;
+  *(int *)   (function + 4) = ((unsigned long) data >> 16) & 0xffffffff;
+  *(short *) (function + 8) = ((unsigned long) data >> 48) & 0xffff;
+  *(short *) (function +10) = 0xB848;
+  *(int *)   (function +12) = (unsigned long) address & 0xffffffff;
+  *(int *)   (function +16) = ((unsigned long) address >> 32) & 0xffffffff;
+  *(short *) (function +20) = 0xE0FF;
+#define is_tramp(function)  \
+  *(unsigned short *) (function + 0) == 0xBA49 && \
+  *(unsigned short *) (function +10) == 0xB848 && \
+  *(unsigned short *) (function +20) == 0xE0FF
+#define hilo(hiword,loword)  \
+  (((unsigned long) (hiword) << 32) | (unsigned long) (loword))
+#define himidlo(hishort,midword,loshort)  \
+  (((unsigned long) (hishort) << 48) | (unsigned long) (midword) << 16 \
+   | (unsigned long) (loshort))
+#define tramp_address(function)  \
+  hilo(*(unsigned int *) (function +16), *(unsigned int *) (function +12))
+#define tramp_data(function)  \
+  himidlo(*(unsigned short *) (function + 8), \
+          *(unsigned int *)   (function + 4), \
+          *(unsigned short *) (function + 2))
+#endif
+#ifdef __s390__
+  /* function:
+
+        bras    %r1,.LTN0_0
+.LT0_0:
+.LC0:
+        .long   0x73554711
+.LC1:
+        .long   0xbabebec0
+.LTN0_0:
+        l       %r0,.LC0-.LT0_0(%r1)
+        l       %r1,.LC1-.LT1_0(%r1)
+        br      %r1
+  */
+  /* What about big endian / little endian ?? */
+  *(unsigned int *)   (function + 0) = 0xA7150006;
+  *(unsigned int *)   (function + 4) = (unsigned int) data;
+  *(unsigned int *)   (function + 8) = (unsigned int) address;
+  *(unsigned int *)   (function +12) = 0x58001000;
+  *(unsigned int *)   (function +16) = 0x58101004;
+  *(unsigned short *) (function +20) = 0x07f1;
+#define is_tramp(function)  \
+  *(int *)            (function + 0) == 0xA7150006 && \
+  *(int *)            (function +12) == 0x58001000 && \
+  *(int *)            (function +16) == 0x58101004 && \
+  *(short *)          (function +20) == 0x07f1
+#define tramp_address(function)  \
+  *(unsigned int *) (function +8)
+#define tramp_data(function)  \
+  *(unsigned int *) (function +4)
 #endif
   /*
    * data:
@@ -1036,7 +1094,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
    * cache. The freshly built trampoline is visible to the data cache, but not
    * maybe not to the instruction cache. This is hairy.
    */
-#if !(defined(__hppanew__) || defined(__rs6000aix__) || defined(__ia64__))
+#if !(defined(__hppanew__) || defined(__powerpcaix__) || defined(__powerpc64aix__) || defined(__ia64__))
   /* Only needed if we really set up machine instructions. */
 #ifdef __i386__
 #if defined(_WIN32)
@@ -1119,7 +1177,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
 #ifdef __arm__
   /* This CPU does not have a separate instruction cache. (I think.) */
 #endif
-#ifdef __rs6000__
+#if defined(__powerpc__) && !defined(__powerpc64__)
   __TR_clear_cache(function);
 #endif
 #ifdef __m88k__
@@ -1134,12 +1192,7 @@ __TR_function alloc_trampoline_r (address, data0, data1)
   return (__TR_function) (function + TRAMP_BIAS);
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void free_trampoline_r (__TR_function function)
-#else
-void free_trampoline_r (function)
-  __TR_function function;
-#endif
 {
 #if TRAMP_BIAS
   function = (__TR_function)((char*)function - TRAMP_BIAS);
@@ -1152,12 +1205,7 @@ void free_trampoline_r (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 int is_trampoline_r (void* function)
-#else
-int is_trampoline_r (function)
-  void* function;
-#endif
 {
 #if defined(is_tramp) && defined(tramp_data)
 #ifdef __hppanew__
@@ -1174,12 +1222,7 @@ int is_trampoline_r (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 __TR_function trampoline_r_address (void* function)
-#else
-__TR_function trampoline_r_address (function)
-  void* function;
-#endif
 {
 #ifdef tramp_address
   return (__TR_function)(tramp_address(((char*)function - TRAMP_BIAS)));
@@ -1188,12 +1231,7 @@ __TR_function trampoline_r_address (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void* trampoline_r_data0 (void* function)
-#else
-void* trampoline_r_data0 (function)
-  void* function;
-#endif
 {
 #ifdef tramp_data
   return ((void**)((char*)function-TRAMP_BIAS+TRAMP_LENGTH))[0];
@@ -1202,12 +1240,7 @@ void* trampoline_r_data0 (function)
 #endif
 }
 
-#if defined(__STDC__) || defined(__GNUC__) || defined(__cplusplus)
 void* trampoline_r_data1 (void* function)
-#else
-void* trampoline_r_data1 (function)
-  void* function;
-#endif
 {
 #ifdef tramp_data
   return ((void**)((char*)function-TRAMP_BIAS+TRAMP_LENGTH))[1];
